#import controlFunc as controlNet
from SpikingNeuralNets import ControlClass
from numpy import array, hstack, linspace, pi
from numpy import zeros, cos, sin, arange, around
from scipy.integrate import odeint
from matplotlib import pyplot as plt
from matplotlib import animation
from matplotlib.patches import Rectangle
from sympy import symbols
from sympy.physics.mechanics import *
from numpy import array, hstack, zeros, linspace, pi, ones
import os
import math
import decimal
import numpy as np
from pyNN.utility.plotting import Figure, Panel
import cPickle as pickle
import gc
import pyNN.nest as sim
from nealCoverClass import NealCoverFunctions
import nealParams as nealParameters

sim.setup(timestep=nealParameters.DELAY,
            min_delay=nealParameters.DELAY,
            max_delay=39.0, debug=0)

##main
def pause():
    programPause = raw_input("Press the <ENTER> key to continue...")

def animate_pendulum(t, states, length, filename=None):
    """Animates the n-pendulum and optionally saves it to file.

    Parameters
    ----------
    t : ndarray, shape(m)
        Time array.
    states: ndarray, shape(m,p)
        State time history.
    length: float
        The length of the pendulum links.
    filename: string or None, optional
        If true a movie file will be saved of the animation. This may take some time.

    Returns
    -------
    fig : matplotlib.Figure
        The figure.
    anim : matplotlib.FuncAnimation
        The animation.

    """
    # the number of pendulum bobs
    numpoints = states.shape[1] / 2

    # first set up the figure, the axis, and the plot elements we want to animate
    fig = plt.figure()

    # some dimesions
    cart_width = 0.4
    cart_height = 0.2

    # set the limits based on the motion
    xmin = around(states[:, 0].min() - cart_width / 2.0 - .3, 1)
    xmax = around(states[:, 0].max() + cart_width / 2.0 + .3, 1)

    # create the axes
    ax = plt.axes(xlim=(xmin, xmax), ylim=(-1.1, 1.1), aspect='equal')

    # display the current time
    time_text = ax.text(0.04, 0.9, '', transform=ax.transAxes)

    # create a rectangular cart
    rect = Rectangle([states[0, 0] - cart_width / 2.0, -cart_height / 2],
                     cart_width, cart_height, fill=True, color='red', ec='black')
    ax.add_patch(rect)

    # blank line for the pendulum
    line, = ax.plot([], [], lw=2, marker='o', markersize=6)

    # initialization function: plot the background of each frame
    def init():
        time_text.set_text('')
        rect.set_xy((0.0, 0.0))
        line.set_data([], [])
        return time_text, rect, line,

    # animation function: update the objects
    def animate(i):
        time_text.set_text('time = {:2.2f}'.format(t[i]))
        rect.set_xy((states[i, 0] - cart_width / 2.0, -cart_height / 2))
        x = hstack((states[i, 0], zeros((numpoints - 1))))
        y = zeros((numpoints))
        #Writer = animation.writers['ffmpeg']
        #writer = Writer(fps=15, metadata=dict(artist='Me'), bitrate=1800)
        
        for j in arange(1, numpoints):
            # x[j] = x[j - 1] + length * cos(states[i, j])
            # y[j] = y[j - 1] + length * sin(states[i, j])
            x[j] = x[j - 1] - length * sin(states[i, j])
            y[j] = y[j - 1] + length * cos(states[i, j])
        line.set_data(x, y)
        return time_text, rect, line,
    
        # call the animator function
    anim = animation.FuncAnimation(fig, animate, frames=len(t), init_func=init,
                                   interval=t[-1] / len(t) * 1000, blit=False, repeat=False)
    anim.save('neuralpole.mp4')
    plt.show()

def Controller(x):
    n=1  
    q = dynamicsymbols('q:' + str(n + 1))  # Generalized coordinates
    v = dynamicsymbols('v:' + str(n + 1))  # Generalized velocities
    K=[-1, 12.15154198, -1.90820654, 3.96320796]

    equilibrium_point = hstack((0, pi / 2 * ones(len(q) - 1), zeros(len(v))))    
    F=0.0
    for i in range(0,4):
        F+=K[i]*(equilibrium_point[i]-x[i])             #K is not a calculation using the mass of cart and pole above, it was found from the other file
        #F+=F2    
    return F

def displayX(Net):
    print Net
    print "X0"
    [WhichInNeuronsFired]=Net.DetermineWhichFired(Net.XPopsInpCells[0], 1, 0)
    x0=Net.CalculateAndPrintForcePop(Net.XPopsInpCells[0], Net.Xincrements[0], Net.numNeuronsInEachXPop[0], 1, rad, WhichInNeuronsFired, runDur, runs, 0)

    print "X1"
    [WhichInNeuronsFired]=Net.DetermineWhichFired(Net.XPopsInpCells[1], 1, 0)
    x1=Net.CalculateAndPrintForcePop(Net.XPopsInpCells[1], Net.Xincrements[1], Net.numNeuronsInEachXPop[1], 1, rad, WhichInNeuronsFired, runDur, runs, 0)


    print "X2"
    [WhichInNeuronsFired]=Net.DetermineWhichFired(Net.XPopsInpCells[2], 1, 0)
    x2=Net.CalculateAndPrintForcePop(Net.XPopsInpCells[2], Net.Xincrements[2], Net.numNeuronsInEachXPop[2], 1, rad, WhichInNeuronsFired, runDur, runs, 0)


    print "X3"
    [WhichInNeuronsFired]=Net.DetermineWhichFired(Net.XPopsInpCells[3], 1, 0)
    x3=Net.CalculateAndPrintForcePop(Net.XPopsInpCells[3], Net.Xincrements[3], Net.numNeuronsInEachXPop[3], 1, rad, WhichInNeuronsFired, runDur, runs, 0)


def initialiseCells(Network, Network2, Network3, numdepths, networkNum):
    voltage=-51.0
    rest=-65.0
    voltlist=ones(numdepths)*rest
    voltlist[0]=voltage

    if(networkNum==1):
        Network.PulseCellPop.initialize(v=voltlist)
    elif(networkNum==2):
        Network.PulseCellPop.initialize(v=voltlist)
        Network2.PulseCellPop.initialize(v=voltlist)        
    else:
        Network.PulseCellPop.initialize(v=voltlist)
        Network2.PulseCellPop.initialize(v=voltlist)
        Network3.PulseCellPop.initialize(v=voltlist)
    
    Network.initializeCellsUsingValue(Network.dt0cell,Network.dt0increment, Network.dt0increment, voltage, Network.numNeuronsDt0Pop,rest)
    Network.initializeCellsUsingValue(Network.dt1cell,Network.dt1increment, Network.dt1increment, voltage, Network.numNeuronsDt1Pop,rest)

def ConnectNetworks(Network1, Network2, minWeight, sim):
    #EXHconnections=[]
    """
    d=Network1.numNeuronsInEachXPop[0]
        
    for i in range(0, d[0]):
        EXHconnections+= [(i,i,minWeight,nealParameters.DELAY)]
        
    connList = sim.FromListConnector(EXHconnections)
    exc_connections=sim.Projection(Network1.X0NextStateCells17, Network2.XPopsInpCells[0], connList, receptor_type='excitatory')
    """
    Network1.summationTwoPopsV3(Network1.X0PopsInpCells1, Network1.RoundDX0dtPopCells, Network2.XPopsInpCells[0], Network2.Xincrements[0], Network1.RoundDX0dtincrements, Network2.Xincrements[0], Network2.numNeuronsInEachXPop[0], Network2.numNeuronsInEachXPop[0], Network1.numNeuronsInRoundDX0dtPop, minWeight, Network1.X0PlusDX0dt0InterSumCells,1,1,Network1.numNeuronsX0PlusDX0dt0InterPop, 37.0)

    """
    EXHconnections=[]
    d=Network1.numNeuronsInEachXPop[1]
        
    for i in range(0, d[0]):
        EXHconnections+= [(i,i,minWeight,nealParameters.DELAY)]
        
    connList = sim.FromListConnector(EXHconnections)
    exc_connections=sim.Projection(Network1.X1NextStateCells17, Network2.XPopsInpCells[1], connList, receptor_type='excitatory')
    """
    
    Network1.summationTwoPopsV3(Network1.X1PopsInpCells1, Network1.RoundDX1dtPopCells, Network2.XPopsInpCells[1], Network2.Xincrements[1], Network1.RoundDX1dtincrements, Network2.Xincrements[1], Network2.numNeuronsInEachXPop[1], Network2.numNeuronsInEachXPop[1], Network1.numNeuronsInRoundDX1dtPop, minWeight, Network1.X1PlusDX1dt1InterSumCells,1,1,Network1.numNeuronsX1PlusDX1dt1InterPop, 37.0)

    """
    EXHconnections=[]
    d=Network1.numNeuronsInEachXPop[2]
        
    for i in range(0, d[0]):
        EXHconnections+= [(i,i,minWeight,nealParameters.DELAY)]

    connList = sim.FromListConnector(EXHconnections)
    exc_connections=sim.Projection(Network1.X2NextStateCells5, Network2.XPopsInpCells[2], connList, receptor_type='excitatory')
    """
#    Network1.summationTwoPopsV3(Network1.X2PopsInpCells14, Network1.RoundDX2dtPopCells, Network2.XPopsInpCells[2], Network2.Xincrements[2], Network1.RoundDX2dtincrements, Network2.Xincrements[2], Network2.numNeuronsInEachXPop[2], Network2.numNeuronsInEachXPop[2], Network1.numNeuronsInRoundDX2dtPop, minWeight, Network1.X2PlusDX2dt2InterSumCells,1,1,Network1.numNeuronsX2PlusDX2dt2InterPop, 13.0)
    Network1.summationTwoPopsV3(Network1.X2PopsInpCells1, Network1.RoundDX2dtPopCells, Network2.XPopsInpCells[2], Network2.Xincrements[2], Network1.RoundDX2dtincrements, Network2.Xincrements[2], Network2.numNeuronsInEachXPop[2], Network2.numNeuronsInEachXPop[2], Network1.numNeuronsInRoundDX2dtPop, minWeight, Network1.X2PlusDX2dt2InterSumCells,1,1,Network1.numNeuronsX2PlusDX2dt2InterPop, 13.0)


#    Network1.summationTwoPopsV3(Network1.X3PopsInpCells20, Network1.RoundDX3dtPopCells, Network2.XPopsInpCells[3], Network2.Xincrements[3], Network1.RoundDX3dtincrements, Network2.Xincrements[3], Network2.numNeuronsInEachXPop[3], Network2.numNeuronsInEachXPop[3], Network1.numNeuronsInRoundDX3dtPop, minWeight, Network1.X3PlusDX3dt3InterSumCells,1,1,Network1.numNeuronsX3PlusDX3dt3InterPop, 1.0)
    Network1.summationTwoPopsV3(Network1.X3PopsInpCells1, Network1.RoundDX3dtPopCells, Network2.XPopsInpCells[3], Network2.Xincrements[3], Network1.RoundDX3dtincrements, Network2.Xincrements[3], Network2.numNeuronsInEachXPop[3], Network2.numNeuronsInEachXPop[3], Network1.numNeuronsInRoundDX3dtPop, minWeight, Network1.X3PlusDX3dt3InterSumCells,1,1,Network1.numNeuronsX3PlusDX3dt3InterPop, 1.0)

networkNum=3

useforce=0
test=0
test2=test
depth=22    ##23 due to the insertion of the pop prior to cos and sin
numdepths=depth*networkNum
##########
#minWeight=0.713
minWeight=0.712464
#########
useNeuralControl=0
print "NETWORK 1"
dt=0.01


if(networkNum==1):
    record1=1
    Control=ControlClass(sim, numdepths, 0, 1, test, dt, record1, useforce,useNeuralControl)  #1==network1
elif(networkNum==2):
    record1=1
    Control=ControlClass(sim, numdepths, 0, 1, test, dt, record1, useforce,useNeuralControl)  #1==network1
    print "NETWORK 2"
    record2=0
    Control2=ControlClass(sim, numdepths, depth, 2, test, dt, record2, useforce,useNeuralControl) #2==network2 #depth=the pulse id associated to the start of each network
else:
    record1=1
    Control=ControlClass(sim, numdepths, 0, 1, test, dt, record1, useforce,useNeuralControl)  #1==network1
    print "NETWORK 3"
    record2=0
    Control2=ControlClass(sim, numdepths, depth, 2, test, dt, record2, useforce,useNeuralControl) #2==network2 #depth=the pulse id associated to the start of each network  
    record3=0
    Control3=ControlClass(sim, numdepths, depth*2, 3, test, dt, record3, useforce,useNeuralControl) #2==network2 #depth=the pulse id associated to the start of each network


#Control.setuprecordings()
#adfadf
#print Control.test
#Control.test=0
#print Control.test
#adfadf
#F=0
#voltage=3
#rest=3
#adfa
#x0=distance, x1=angle, x2=velocity of cart, x3=velocity of pendulum
#dx0=velocity of cart, dx1=velocity of pendulum, dx2, acceleration of cart, dx3=acceleration of pendulum
    #acceleration*dt=velocity, velocity/dt=acceleration
    
if(useforce==1):
    #x=[-1.46, 0.53, -11.63, -2.73]
    x=[0.5, 0.83, -3.08, 1.14] ##to use with force as force ranges from -1.13 to 0.2
else:
    x=[0,0,1.0,1.0]
#x=[1.3, 9.26, -4.93, 9.64]
#x=[0.98, 9.19, -4.55, 9.47]
#x=[3.49, 23.48, 1.42, 7.19] #0.04 off
#x=[1.33, 11.14, 1.42, 5.38]
#x=[0.9, 4.09, -0.02, 6.0]
#x=[-17.94, -0.24, -0.04, 0.98]

#x=[-18.35, -0.22, 0.24, 0.95]
#x=[-7.0, -0.2, -11.65, -1.96]
#x=[-2.03, 0.38, -12.71, -2.94]    
#x=[1.06, 1.89, 1.0, 5.05]
#############################
#x=[-0.91, 0.63, -11.2, -2.69]
############################
#x=[-1.9, 0.45, -11.93, -2.76]
#x=[0.83, 3.51, -1.8, 5.97] ###do this one
#x=[0.9, 2.0, 0.2, 4.5]
#x=[1.4000000000000001, 30.0, 4.2999999999999998, 0]
#x=[1.4, 30.0, 4.3, 0]
#x=[-0.2, 29.7, -0.3, 5.7]
#x=[0.4, 3.8, -4.0, 7.0]
#x=[0.9, 1.6, 0.2, 4.5]
#####  5.1
#x=[0.9, 5.0, -4.0, 7.0]
#x=[0.9, 1.6, 0.2, 4.5]  #0.9, 2.0, -0.9, 5.4==answer
#x=[0.9, 1.6, 0.2, 4.5]
#x=[0.9, 1.6, 0.2, 4.5]
##########
#x=[0.9, 1.3, 0.9, 3.8]
##########
#x=[0.9,2.0,-0.9,5.4]
#x=[0.9, 1.6, 0.2, 4.5]
k=x
xtest=x
xtest2=x
############################
"""
xstates=[]
loops=50
sec=(100*loops)/1000
t = linspace(0, sec, loops)
l=1
print t
#print xstates


for i in range(0,loops):
    #print("Real time: %g ms, simulation time: %g ms"  % (start, start2))
    if(i>=0):
        #F=Controller(k)
        F=0.0
        k=Control.testforward(k, F)
            
        print "test values"
        print k
        print "F"
        print F
    else:
        print "test values"
        print k

    xstates.append(k)

xstates=np.array(xstates)

print xstates
#print xstates.shape
#print xstates.shape[1]/2
animate_pendulum(t, xstates, l)

adfadf
"""
#################################
#print "hello"
print Control.XPopsInpCells[0]
print Control.numNeuronsInEachXPop
#x=[0.4, 0.4, 1.0, 1.3]
#x=[0.8,1.0,1.0,3.1]

#d=Control.testforward(x)
#print d
#adfa




#print Control.numNeuronsInEachXPop
#print Control2.numNeuronsInEachXPop
#Control.numNeuronsInEachXPop=[3,3]
#print Control.numNeuronsInEachXPop
#print Control2.numNeuronsInEachXPop
#adfadf

#print Control2.XPopsInpCells[2]
print Control.Xincrements[2]
print Control.numNeuronsInEachXPop[2]

#print x     #[-0.2, 30.2, 1.6, 4.6]
#x=[-0.20000000000000001, 29.6999999999999993, -0.30000000000000004, 5.7000000000000002]
#k=Control.testforward(x)
#print k
#adfafd

if(networkNum==1):
    print "CREATING NETWORK 1"
    Results=Control.CreateNetwork()
elif(networkNum==2):
    print "CREATING NETWORK 1"
    Results=Control.CreateNetwork()
    print "CREATING NETWORK 2"
    Results2=Control2.CreateNetwork()
else:
    print "CREATING NETWORK 1"
    Results=Control.CreateNetwork()
    print "CREATING NETWORK 2"
    Results2=Control2.CreateNetwork()    
    print "CREATING NETWORK 3"
    Results3=Control3.CreateNetwork()

if(networkNum==1):
    initialiseCells(Control, 0, 0, numdepths, networkNum)
elif(networkNum==2):
    initialiseCells(Control, Control2, 0, numdepths, networkNum)
else:
    initialiseCells(Control, Control2, Control3, numdepths, networkNum)
#initialiseCells(Control2, numdepths)

start2=0


x0states=[]
x1states=[]
x2states=[]
x3states=[]

xstates=[]
simtime=10



        #runDur = 50.0
        #runs=2
voltage=-51.0
rest=-65.0
runDur=44.0
runs=0
rad=0
####CONNECT BOTH NETWORKS
###CONNECT NETWORK 1 TO NETWORK 2
if(networkNum==1):
    ConnectNetworks(Control, Control, minWeight,sim)
elif(networkNum==2):
    ConnectNetworks(Control, Control2, minWeight,sim)
#########CONNECT NETWORK 2 TO NETWORK 1
    ConnectNetworks(Control2, Control, minWeight,sim)    
else:
    ConnectNetworks(Control, Control2, minWeight,sim)
#########CONNECT NETWORK 2 TO NETWORK 1
    ConnectNetworks(Control2, Control3, minWeight,sim)

    ConnectNetworks(Control3, Control, minWeight,sim)


######INITIALISE NETWORK
numsets=4
for j in range(0,numsets):

    Control.initializeCellsUsingValue(Control.XPopsInpCells[j],x[j], Control.Xincrements[j], voltage, Control.numNeuronsInEachXPop[j],rest) #this calculates which cells to initialise when given a value
    #Control2.initializeCellsUsingValue(Control2.XPopsInpCells[j],0, Control2.Xincrements[j], voltage, Control2.numNeuronsInEachXPop[j],rest) #this calculates which cells to initialise when given a value


#    if(Control.Earray[j]!=0.0):
#        Control.initializeCellsUsingValue(Control.EPopsInpCells[j],Control.Earray[j], Control.Eincrements[j], voltage, Control.numNeuronsInEachEPop[j],rest)

        
        #self.testCells()
        #sdfafd
        #
        #self.initializeCellsUsingValue(ControlClass.XPopsInpCells[1],rad, ControlClass.Xincrements[1], voltage, ControlClass.numNeuronsInEachXPop[1],rest) #this calculates which cells to initialise when given a value
        #self.initializeCellsUsingValue(ControlClass.XPopsInpCells[3],0.9, ControlClass.Xincrements[3], voltage, ControlClass.numNeuronsInEachXPop[3],rest) #this calculates which cells to initialise when given a value


        #[WhichOutNeuronsFired]=self.DetermineWhichFired(ControlClass.SinPopCells,1,0)

        
        ###FORCE
        ####
#F=Controller(x)
F=0.0
Control.initializeCellsUsingValue(Control.FOutCells,F, Control.Fincrements, voltage, Control.numNeuronsInFPop ,rest) #this calculates which cells to initialise when given a value
#####
#start=1.0  
#####
start=15.0 #need to initiate x and pulse cells

#start=10
#sim.run_until(start)
#Control.testSINCOS()
#adfadf
if(useforce==1):
    #duration=0.1
    duration=4.0  #4.0
else:
    duration=4.0

loops=duration/dt
loops=int(loops)
pauseflag=0
Network=1
Net=Control
flag=1
timeF=14.0
###flag=1 run simulation up until +14ms to inject F values
###flag=0 run sim up until end of loop, i.e. 44ms
#test=0
for i in range(0,loops*2): ##*2 as we are looping through the simulation and initiating the F values
    print "in loop"
    print i
    print start
    #xtest=[0.9, 1.6, 0.2, 4.5]
    #xtest2=[0.9, 1.6, 0.2, 4.5]
    #xtest=[0.9,2.0,-0.9,5.4]
    if(flag==0):
        print("Real time: %g ms, simulation time: %g ms"  % (start, start2))
        """
    if(i>=2):
        #if(i==10):
         #   Control.setuprecordings()
          #  Control2.setuprecordings()
        Control.test=1
        Control2.test=1
        test=1
        """
        print "i"
        print i
        print "flag"
        print flag    
        print "input"
        print xtest
#    print"#####"



        
    #F=0.015
    if(flag==0):
        if(useforce==1):
            if(useNeuralControl==0):
                F=Controller(xtest)

            else:
                [WhichInNeuronsFired]=Net.DetermineWhichFired(Net.FOutCells2, 1, 0)
                F=Net.CalculateAndPrintForcePop(Net.FOutCells2, Net.Fincrements, Net.numNeuronsInEachF2Pop, 1, rad, WhichInNeuronsFired, runDur, runs, 0)
                
            Net.initializeCellsUsingValue(Net.FOutCells,F, Net.Fincrements, voltage, Net.numNeuronsInFPop ,rest) #this calculates which cells to initialise when given a value
            
                

    #if(i==1):
    #    start=15.0  

    #if(flag==0):
    #    Net.test=1
    #    test=1
    print "start run"
    sim.run_until(start)
    print "end run"
    
    """
    if(i<2):
        Control.ClearData()
        Control2.ClearData()
    """

    #if((flag==0)and(i>0)):
    #    print "TEST F NETWORK"
    #    Net.testF()
    #    #adfadf

    if((flag==0)):                ##keep with network 1 until i==2 as at i==0 we set x values and pulse neuronns and i==1 (i.e.16ms) we initiatlie the F neurons
        if(networkNum==1):
            Net=Control
            #print Net
            Network=1
        elif(networkNum==2):
            if(Network==1):     ##alternate the networks we extract the results fromm
                #if(test==1):
                print "NETWORK1"
                Net=Control2
                #print Net
                Network=2
            elif(Network==2):
                #if(test==1):
                print "NETWORK2"
                Net=Control
                #print Net
                Network=1
        else:   
            if(Network==1):     ##alternate the networks we extract the results fromm
                #if(test==1):
                print "NETWORK1"
                Net=Control2
                #print Net
                Network=2
            elif(Network==2):
                #if(test==1):
                print "NETWORK2"
                Net=Control3
                #print Net
                Network=3
            elif(Network==3):
                #if(test==1):
                print "NETWORK3"
                Net=Control
                #print Net
                Network=1
      


    if((flag==0)):
        print "input (i>0)"
        print xtest
        print '{0:.16f}'.format(xtest[0])
        print '{0:.16f}'.format(xtest[1])
        print '{0:.16f}'.format(xtest[2])
        print '{0:.16f}'.format(xtest[3])
        #k=Control.testforward(k)
        #copyt=xtest
        #F=Controller(x)
        #F=0.0
        #F=0.015
        p=Control.actualforward(xtest2)
        #p=Control.actualforward(xtest2,F)
        print "actual results"
        print p
        #copyt2=xtest
        #k=Control.testforward(xtest,F)
        k=Control.testforward(xtest, F, dt, i)
        print "################START"
        print "F"
        print F
        print "test values"
        print k        
    else:
        if(i==0):
            print "test values"
            print k 



    #if(Network==1):
    #    print "NETWORK1"
    #elif(Network==2):
    #    print "NETWORK2"
    #else:
   #     print "NETWORK3"

    #if(i==0):
    #    [WhichInNeuronsFired]=Net.DetermineWhichFired(Net.XPopsInpCells[0], 1, 0)#clear network 1 spiking data
            
    if((flag==0)or(i==0)):
        [WhichInNeuronsFired]=Net.DetermineWhichFired(Net.XPopsInpCells[0], 1, 0)
        x0=Net.CalculateAndPrintForcePop(Net.XPopsInpCells[0], Net.Xincrements[0], Net.numNeuronsInEachXPop[0], 1, rad, WhichInNeuronsFired, runDur, runs, 0)
        if(test==1):
            print "X0"
            print "valueX0"
            print x0
        x0states+=[x0]

    #print "ControlX0"
    #[WhichInNeuronsFired]=Net.DetermineWhichFired(Control.XPopsInpCells[0], 1, 0)
    #x0=Control.CalculateAndPrintForcePop(Control.XPopsInpCells[0], Control.Xincrements[0], Control.numNeuronsInEachXPop[0], 1, rad, WhichInNeuronsFired, runDur, runs, 0)

    #print "Control2X0"
    #[WhichInNeuronsFired]=Control2.DetermineWhichFired(Control2.XPopsInpCells[0], 1, 0)
    #x0=Control2.CalculateAndPrintForcePop(Control2.XPopsInpCells[0], Control2.Xincrements[0], Control2.numNeuronsInEachXPop[0], 1, rad, WhichInNeuronsFired, runDur, runs, 0)

    #adfadf

    
        [WhichInNeuronsFired]=Net.DetermineWhichFired(Net.XPopsInpCells[1], 1, 0)
        x1=Net.CalculateAndPrintForcePop(Net.XPopsInpCells[1], Net.Xincrements[1], Net.numNeuronsInEachXPop[1], 1, rad, WhichInNeuronsFired, runDur, runs, 0)
        if(test==1):
            print "X1"
            print "valueX1"
            print x1
        x1states+=[x1]



    
    #print Net.XPopsInpCells[2]
    #print Net.Xincrements[2]
    #print Net.numNeuronsInEachXPop[2]    
        [WhichInNeuronsFired]=Net.DetermineWhichFired(Net.XPopsInpCells[2], 1, 0)
        x2=Net.CalculateAndPrintForcePop(Net.XPopsInpCells[2], Net.Xincrements[2], Net.numNeuronsInEachXPop[2], 1, rad, WhichInNeuronsFired, runDur, runs, 0)
        if(test==1):
            print "X2"
            print "valueX2"
            print x2
        x2states+=[x2]    

    
        [WhichInNeuronsFired]=Net.DetermineWhichFired(Net.XPopsInpCells[3], 1, 0)
        x3=Net.CalculateAndPrintForcePop(Net.XPopsInpCells[3], Net.Xincrements[3], Net.numNeuronsInEachXPop[3], 1, rad, WhichInNeuronsFired, runDur, runs, 0)
        if(test==1):
            print "X3"
            print "valueX3"
            print x3
        x3states+=[x3]

    #print "OTHER"
    #displayX(Control2)


            #print "x0"
            #print x0states
        xstates.append([x0,x1,x2,x3])

        print "Neural values"
    #xtest2=[x0,x1,x2,x3]
    #print xtest2
    #print("Neural states are X0=  %g X1=  %g X2=  %g X3=  %g " % (x0states[i], x1states[i], x2states[i], x3states[i],))

    #print "before round"
    #print '{0:.16f}'.format(x0)
    #print '{0:.16f}'.format(x1)
    #print '{0:.16f}'.format(x2)
    #print '{0:.16f}'.format(x3)    

    #if(x0<0):
    #    adddec=-0.000001
    #elif(x0>0):
    #    adddec=0.000001
   # else:
   #     adddec=0.0
        

    #f="{0:.6f}".format(x0)
    #x0=float(f)+0.0
    #f="{0:.6f}".format(x1)
   # x1=float(f)+0.0
   # f="{0:.6f}".format(x2)
   # x2=float(f)+0.0
   # f="{0:.6f}".format(x3)
    #x3=float(f)+0.0    
        copyx0=round(x0,5)
        copyx1=round(x1,5)
        copyx2=round(x2,5)
        copyx3=round(x3,5)
        copy2x0=round(x0,5)
        copy2x1=round(x1,5)
        copy2x2=round(x2,5)
        copy2x3=round(x3,5)    
        xtest=[copyx0,copyx1,copyx2,copyx3]
        xtest2=[copy2x0,copy2x1,copy2x2,copy2x3]

        print xtest
    #print "#######################"
    #print "test36"
    #print '{0:.16f}'.format(x0)
    #print '{0:.16f}'.format(x1)
    #print '{0:.16f}'.format(x2)
    #print '{0:.16f}'.format(x3)
            

    

        if((test==1)and(flag==0)):
            print "in dx test"
            print i
            #if((i==6)or(i==11)):
            print "test dx2"########look at previous network results to identify how the results in second network were gained
            if(Net==Control):
                print "NETWORK3"
                #Control3.testF()
                Control3.testDX0()
            elif(Net==Control2):
                print "NETWORK1"
                #Control.testF()
                Control.testDX0()
            else:
                print "NETWORK3"
                #Control2.testF()
                Control2.testDX0()            
            

        error=zeros(4)
        for kk in range(0,4):
            d=k[kk]-xtest[kk]
            error[kk]=d
            if(d>0):
                print "error in result %g" %(kk)
                if((d>0.02)or(d<-0.02)):
                    pauseflag=1
            #print xtest
            #print k
            #print error
            #self.pause()
                print "error"
                print error

        print "error"
        print error
        if(pauseflag==1):
            #pause()
            pauseflag=0
        print "##########################END"
 ############ROUND DOEST WORK.  IT ONLY WORKS AT THE ISTANCE U USSE IT. IT DOESNT CHANGE THE VALUE   
    
    #start+=runDur       
    if(flag==1):
        flag=0
        start+=(runDur-timeF)  ##need run the sim from 1.0 to 16 i.e. just before we initialise the F values at 17ms
    else:
        flag=1
        start+=timeF
        if(i>0):
            start2+=simtime

    print "secondhalf flag"
    print flag   
    

#jhgjhg
#Net.testDX2()
#Control.deletedata()
#Control2.deletedata()
#Control3.deletedata()
#del Control
#del Control2
#del Control3
gc.collect()
l=1
#t = linspace(0, 10, 1000)
#10s=10000ms
#45ms = 222
#100=0.1dt

#p=1.0/dt
#sec=(p*loops)/1000
#t = linspace(0, sec, loops)
t=linspace(0, duration, loops)
print t
#print xstates
xstates=np.array(xstates)
print xstates
#print xstates.shape
#print xstates.shape[1]/2
animate_pendulum(t, xstates, l)
#jkhkjh



#Control.printtest("dave")

#x=[0.3482166, 2.37451689, -2.42897444, 6.7490725]
#for i in range(0,50):

#x=[-2.0, 0.5, 1.0, -1.5]  #=4 with our k and e values

#x=[-2.0, 2.0, 1.0, 0.9]
#x=[0.0, 0.0, 1.0, 1.0]



#x=[0.14003722, 0.1400264, 10.70073464, 10.69774274]
#    x=[-7.0, 0.5, -7.5, 2]  #= -17 with the controllers k and e values
#x=np.multiply(x,10)
#print x
#adfdaf

#Force=Control.ForceOutput(x,0,0)   #segment=0
##radians -pi...pi
Rad=2#-0.5# probs=-2.5      #angle in radians


#t = linspace(0, pi, 100)
#p = linspace(-pi, pi, 200)
#print sin(p)
#print sin(0)    #0
#print sin(1.57) #1
#print sin(3.14) #0
#print sin(4.71) #-1
#print sin(6.28) #0
#plt.plot(sin(p))
#plt.show()
#plt.plot(cos(p))
#plt.show()
#print p
#DONT FORGET TO SET CLEAR DATA IN DETERMINEWHI,CHNEURONS function
"""
internal=1
if(internal==1):
    [SinValue, CosValue, roundmult, actualmult, sumvalue, subvalue, divvalue, rounddivvalue, properdivvalue]=Control.testInternal(Rad, x)


    print "sinvalue"
    print SinValue
    print "cosvalue"
    print CosValue
    print "roundmult (Round(COS*SIN))"
    print roundmult
    print "actualmult (COS*SIN)"
    print actualmult
    print "sumvalue (COS+SIN)"
    print sumvalue
    print "subvalue (COS-SIN)"
    print subvalue
    print "divvalue (1/COS)"
    print divvalue
    print "round SIN/Cos value"
    print rounddivvalue
    print "Sin/Cos value"
    print properdivvalue
else:
    Force=Control.ForceOutput(x,0,0)
    print "outputFORCE"
    print Force
"""

